"Engine Of Love" is a framework for the LÖVE game engine and provides additional functionality:

- management of user preferences
- configurable key bindings for changing screen mode, fullscreen, vsync, fsaa, audio volume, muting audio and debug functions
- scales and centers the screen to any resolution (needs canvas support)
- fps capping
- OSD (on screen display) of any message with customizable appearance
- debug information which can be shown inside the window, user can add a table with own debug parameters, appearance is fully customizable
- provides a boolean variable for easy (un)pausing game if window is (de)focussed

FIRST USE
---------

It's very easy to use the framework, only a few simple steps have to be followed:

- after downloading place the "engineoflove" folder into your game root directory (where your main.lua is)
- add the following line to your main.lua:   
<code>require "engineoflove"</code>

Voila!
All the additional functions are now immediately available to your game.

CONFIGURATION
-------------

There are two files inside the <code>engineoflove</code> directory which are used for configuration purposes: <code>keys.lua</code> and <code>prefs.lua</code>.

Every configuration item has four fields attached to it:

* <code>x</code> -- is this configuration item exposed to the user configuration?
* <code>f</code> -- is this item fixed, means it cannot be altered from the user configuration file?
* <code>v</code> -- the actual value of the settings item
* <code>i</code> -- an info field for a description of the preferences item

When the program is started for the first time, two configuration files are generated, their names are defined in <code>prefs.file.prefs</code> and <code>prefs.file.keys</code>.
They can be found below <code>~/.local/share/love/</code> in the subdirectory which is defined by <code>t.identity</code> in your <code>conf.lua</code> file.
Both files include only the configuration items marked with <code>x=true</code>.

Once the user configuration files are created, their settings items which have been marked with <code>f=false</code> override the internal settings, therefor modifying these items in <code>keys.lua</code> or <code>pregs.lua</code> will have no effect anymore.

For easy separation of your (the game developers) preferences from the framework's defaults, it also looks for the files <code>keys.lua</code> and <code>prefs.lua</code> in your main folder, right there where <code>main.lua</code> resides.   
You can override any default configuration items inside these files, you can simply assign new values, e.g.:

    prefs.screen.canvas_w.v = 320
    prefs.screen.canvas_h.v = 192
    prefs.osd.text_align.f  = true

KEY BINDINGS
------------

All shortkeys are configured in <code>engineoflove/keys.lua</code>.    
To disable a shortkey completely, an empty value <code>v=""</code> can be used.

There are already many keys defined by default, e.g. F1-F6 change screen mode settings, F7-F9 change audio related settings and F11-F12 toggle debug information.
For a full list of shortkeys consult the <code>keys.lua</code> file itself.

SCREEN SCALING
--------------

The canvas size has default values of 512x384 pixels.   
These are reasonable values for displaying pixelart graphics on commonly used screen resolutions.   
For an explanation of this choice please refer to this article by SiENcE: [crankgaming.blogspot.de](http://crankgaming.blogspot.de/2013/01/aeon-of-sands-pyramid-scaling.html).

There are two scaling modes which can be controlled by the <code>prefs.screen.multiscale</code> parameter:

If <code>multiscale</code> is set to true, the canvas will be centered and only scaled to multiple sizes of itself. This assures that pixelart graphics are not distorted and pixels are always square (thanks [SiENcE](https://love2d.org/forums/viewtopic.php?f=5&t=33142#p101760)).   

If it is false, all graphics are scaled to the active window size even if the scale factor is a fractional value.
This will guarantee that the graphics will always fill the whole window, but will cause visible distortions on pixelart graphics.

__Both scaling methods need canvas support.__

To still provide some kind of nice screen setup even without canvas support, offset values are written to <code>eol.screen.ofs_x_nocan</code> and <code>eol.screen.ofs_y_nocan</code>.

These two variables contain 0 in case canvases are used and a nonzero value if canvases are not supported.
If these variables are added to all coordinates in the program, the graphics will still be centered even without canvas support.

Example:

    function love.draw()
        local _x=eol.screen.ofs_x_nocan
        local _y=eol.screen.ofs_y_nocan

        love.graphics.setColor(255,0,0)
        love.graphics.rectangle("fill",_x+0,_y+0,640,480)
    end

FPS CAPPING
-----------
The frame rate of the program is capped to 80fps by default.   
It can be adjusted as desired using the <code>prefs.screen.fpslimit</code> parameter.
A value of 0 (zero) means that capping is disabled and the program runs at maximum fps.   

It's probably a good idea to have fps capping enabled, too many frames waste precious CPU time and the human eye cannot even tell a difference anymore.

OSD (On-Screen-Display)
-----------------------

To show OSD event information just pass any single or multiline string to <code>eol.osd.msg</code>.    

The OSD is displayed with the parameters from the <code>osd</code> section of the preferences.
Nearly every aspect of it is heavily customizable, e.g. colors, transparency, position, alignment and font.

By default screen actions (e.g. resolution changes, toggling of vsync and fullscreen) and audio related events (e.g. volume changes and toggling mute) are shown with OSD, but any string passed to <code>eol.osd.msg</code> will be displayed as well.

DEBUG INFORMATION
-----------------

By default the F11 key toggles the display of debug information.   
Nearly every visual aspects can be controlled in the preferences.

There are two pages available which can be toggled by default with F12:

### system debug information

This page shows system internal values, e.g. framerate, memory consumption, screen and audio settings.

### user debug information

The user can add his own debug information which will be shown here.
As long as the <code>eol.debug.user</code> is empty, this page can not be selected.

This table can be defined e.g. inside love.load() and has the following format:

    eol.debug.user = {
        { n="<name>", f=<function> },
        { n="<name>", f=<function> },
        ...
    }

<code>n</code> is the name of the parameter and <code>f</code> contains a function, it's return value will be shown in the debug information.

Example:

    game_status="ingame"

    eol.debug.user = {
        { n="game status", f=function() return game_status end },
        { n="clock", f=os.clock },
    }

will result in a user debug display similar to:

    game status = ingame
    clock       = 0.51

WINDOW FOCUS
------------
The framework writes to the boolean variable <code>eol.focus</code> if the window is currently focussed or not.   
This makes it easy to pause the game when the window is not focussed, e.g. put in <code>love.update(dt)</code> something similar to this:

    if not eol.focus then game_paused=true end

It would be userfriendly to not automatically unpause the game when the window receives focus, as the player might need some time to prepare.
Instead let the player resume the game through a button or key.

NOTES
-----

### love.run()

The framework overrides the default <code>love.run()</code> function provided by LÖVE 0.8.   
If for any reason <code>love.run()</code> needs to be customized, the existing function in <code>engineoflove/inc/callbacks.lua</code> needs to be modified instead of defining it again.   
It's important that nothing between <code>-- BEGIN EOL</code> and <code>-- END EOL</code> is modified and the position of these blocks are not changed.

### canvas switching

If you have code in your program for canvas switching, then you should remove it.
The framework automagically takes care of it, so your canvas code might interfere with the existing code.

### first call to love.update()

The first call to <code>love.update()</code> after the program starts is actually thrown away.    
If your game loads lots of stuff in the beginning, the first <code>dt</code> value will be pretty big, it can be several seconds or more.
This huge <code>dt</code> value could cause a jump in the beginning of your game depending on how you handle the update code, so I think it's better not to use it.
