--[[
        Engine Of Love - a framework for the LÖVE game engine
        Copyright (C) 2013 vitaminx (vitaminx@callistix.net)

        This program is free software: you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation, either version 3 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program.  If not, see <http://www.gnu.org/licenses/>.
--]]

eol.osd={}

function eol.osd.init()
    -- load font
    if prefs.osd.font_type.v=="pixmap" then eol.osd.font=love.graphics.newImageFont(prefs.osd.font_path.v,eol.charmap)
    elseif prefs.osd.font_type.v=="ttf" then eol.osd.font=love.graphics.newFont(prefs.osd.font_path.v,prefs.osd.font_size.v)
    else eol.osd.font=love.graphics.newFont(prefs.osd.font_size.v) end

    -- calculate horizontal gap inside OSD rectangle
    eol.osd.hgap=(eol.osd.font:getWidth("1")/2)+prefs.osd.gap_off.v

    -- calculate colors for background and font
    eol.osd.color_r,eol.osd.color_g,eol.osd.color_b=eol.misc.convertrgb(prefs.osd.color.v)
    eol.osd.bg_color_r,eol.osd.bg_color_g,eol.osd.bg_color_b=eol.misc.convertrgb(prefs.osd.bg_color.v)
end

function eol.osd.update(_dt)
    if prefs.osd.enabled.v and eol.osd.msg then
        -- check if a new message arrived
        if eol.osd.msg~=eol.osd.msg_old then
            -- reset all values
            eol.osd.trans=1
            eol.osd.msg_old=eol.osd.msg
            eol.osd.timer=prefs.osd.time.v+prefs.osd.fadeout.v
            eol.osd.width=0

            -- calculate dimensions of rectangle
            local _lines=0
            for _l in string.gmatch(eol.osd.msg,"[^\r\n]+") do
                _lines=_lines+1
                if eol.osd.font:getWidth(_l)>eol.osd.width then eol.osd.width=eol.osd.font:getWidth(_l) end
            end
            eol.osd.height=eol.osd.font:getHeight(eol.osd.msg)*_lines

            -- calculate y position of OSD rectangle
            if prefs.osd.valign.v=="bottom" then eol.osd.ypos=prefs.screen.canvas_h.v-eol.osd.height-prefs.osd.voff.v-prefs.osd.height_off.v
            elseif prefs.osd.valign.v=="center" then eol.osd.ypos=((prefs.screen.canvas_h.v/2)-(eol.osd.height/2)+prefs.osd.voff.v)-(prefs.osd.height_off.v/2)
            else eol.osd.ypos=prefs.osd.voff.v end
    
            -- calculate x position of OSD rectangle
            if prefs.osd.halign.v=="right" then eol.osd.xpos=prefs.screen.canvas_w.v-eol.osd.width-prefs.osd.hoff.v-eol.osd.hgap*2
            elseif prefs.osd.halign.v=="center" then eol.osd.xpos=(prefs.screen.canvas_w.v/2)-(eol.osd.width/2)+(prefs.osd.hoff.v-eol.osd.hgap)
            else eol.osd.xpos=prefs.osd.hoff.v end
        else
            -- if no new message arrived then advance timer for old message
            eol.osd.timer=eol.osd.timer-_dt
            if eol.osd.timer<prefs.osd.fadeout.v then
                eol.osd.trans=eol.osd.timer/prefs.osd.fadeout.v
            end
            -- message expited, remove it
            if eol.osd.timer<0 then
                eol.osd.msg=nil
                eol.osd.msg_old=nil
            end
        end
    end
end

function eol.osd.draw()
    if prefs.osd.enabled.v and eol.osd.msg then
        -- draw background rectangle
        love.graphics.setColor(eol.osd.bg_color_r,eol.osd.bg_color_g,eol.osd.bg_color_b,(1-prefs.osd.bg_trans.v)*255*eol.osd.trans)
        love.graphics.rectangle("fill",eol.screen.ofs_x_nocan+eol.osd.xpos,eol.screen.ofs_y_nocan+eol.osd.ypos,eol.osd.width+eol.osd.hgap*2,eol.osd.height+prefs.osd.height_off.v)

        -- draw OSD message
        love.graphics.setFont(eol.osd.font)
        love.graphics.setColor(eol.osd.color_r,eol.osd.color_g,eol.osd.color_b,(1-prefs.osd.trans.v)*255*eol.osd.trans)
        love.graphics.printf(eol.osd.msg,eol.screen.ofs_x_nocan+eol.osd.xpos+eol.osd.hgap,eol.screen.ofs_y_nocan+eol.osd.ypos+prefs.osd.vtext_off.v,eol.osd.width,prefs.osd.text_align.v)
    end
end
